@@ -1,18 +1,16 @@
 import logging
 import re
-import time
-from contextlib import contextmanager
+from contextlib import suppress
 from enum import Enum
-from threading import Thread
+from functools import cached_property
 from typing import Any
 
 import pandas as pd
 import redshift_connector
-from pydantic import Field, SecretStr, root_validator, validator
+from pydantic import Field, SecretStr, create_model, root_validator, validator
 from pydantic.types import constr
 
 from toucan_connectors.common import ConnectorStatus
-from toucan_connectors.connection_manager import ConnectionManager
 from toucan_connectors.redshift.utils import build_database_model_extraction_query, types_map
 from toucan_connectors.sql_query_helper import SqlQueryHelper
 from toucan_connectors.toucan_connector import (
@@ -22,6 +20,7 @@ from toucan_connectors.toucan_connector import (
     TableInfo,
     ToucanConnector,
     ToucanDataSource,
+    strlist_to_enum,
 )
 
 TABLE_QUERY = """SELECT DISTINCT tablename FROM pg_table_def WHERE schemaname = 'public';"""
@@ -51,12 +50,6 @@ ORDERED_KEYS = [
 
 logger = logging.getLogger(__name__)
 
-redshift_connection_manager = None
-if not redshift_connection_manager:
-    redshift_connection_manager = ConnectionManager(
-        name='redshift', timeout=25, wait=0.2, time_between_clean=10, time_keep_alive=60
-    )
-
 
 class AuthenticationMethod(str, Enum):
     DB_CREDENTIALS: str = 'db_credentials'
@@ -89,6 +82,19 @@ class RedshiftDataSource(ToucanDataSource):
     )
     language: str = Field('sql', **{'ui.hidden': True})
 
+    @classmethod
+    def get_form(cls, connector: 'RedshiftConnector', current_config: dict[str, Any]):
+        """
+        Method to retrieve the form with a current config
+        Once the connector is set, we are able to give suggestions for the `database` field
+        """
+        default_db = current_config.get('database', DEFAULT_DATABASE)
+        return create_model(
+            'FormSchema',
+            database=strlist_to_enum('database', connector.available_dbs, default_db),
+            __base__=cls,
+        ).schema()
+
 
 class RedshiftConnector(ToucanConnector, DiscoverableConnector):
     data_source_model: RedshiftDataSource
@@ -129,15 +135,19 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
     session_token: str | None = Field(None, description='Your session token')
     profile: str | None = Field(None, description='AWS profile')
     region: str | None = Field(None, description='The region in which there is your aws account.')
-    _is_alive: bool = True
 
     class Config:
         underscore_attrs_are_private = True
+        keep_untouched = (cached_property,)
 
         @staticmethod
         def schema_extra(schema: dict[str, Any]) -> None:
             schema['properties'] = {k: schema['properties'][k] for k in ORDERED_KEYS}
 
+    @cached_property
+    def available_dbs(self) -> list[str]:
+        return self._list_db_names()
+
     @validator('host')
     def host_validator(cls, v):
         return re.sub(r'^https?://', '', v)
@@ -166,10 +176,6 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
             raise ValueError(AuthenticationMethodError.UNKNOWN.value)
         return values
 
-    @staticmethod
-    def get_redshift_connection_manager() -> ConnectionManager:
-        return redshift_connection_manager
-
     def _get_connection_params(self, database) -> dict[str, Any]:
         con_params = dict(
             database=database,
@@ -199,49 +205,17 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
 
     def _get_connection(self, database) -> redshift_connector.Connection:
         """Establish a connection to an Amazon Redshift cluster."""
-
-        def connect_function() -> redshift_connector.Connection:
-            con = redshift_connector.connect(
-                **self._get_connection_params(
-                    database=database if database else None,
-                ),
-            )
-            con.autocommit = True  # see https://stackoverflow.com/q/22019154
-            con.paramstyle = 'pyformat'
-            return con
-
-        def alive_function(connection) -> bool:
-            logger.info(f'Alive Redshift connection: {connection}')
-            return self._is_alive
-
-        def close_function(connection) -> None:
-            logger.info('Close Redshift connection')
-            if not self._is_alive:
-                return connection.close()
-
-        connection: redshift_connector.Connection = redshift_connection_manager.get(
-            identifier=f'{self.get_identifier()}{database}{self.cluster_identifier}',
-            connect_method=connect_function,
-            alive_method=alive_function,
-            close_method=close_function,
-            save=True if database else False,
+        con = redshift_connector.connect(
+            **self._get_connection_params(
+                database=database if database else None,
+            ),
         )
-        if self.connect_timeout is not None:
-            t = Thread(target=self.sleeper)
-            t.start()
-        return connection
-
-    def sleeper(self):
-        time.sleep(self.connect_timeout)
-        self._is_alive = False
-
-    @contextmanager
-    def _get_cursor(self, database) -> redshift_connector.Cursor:
-        with self._get_connection(database=database) as conn, conn.cursor() as cursor:
-            yield cursor
+        con.autocommit = True  # see https://stackoverflow.com/q/22019154
+        con.paramstyle = 'pyformat'
+        return con
 
     def _retrieve_tables(self, database) -> list[str]:
-        with self._get_cursor(database=database) as cursor:
+        with self._get_connection(database=database).cursor() as cursor:
             cursor.execute(TABLE_QUERY)
             res = cursor.fetchall()
         return [table_name for (table_name,) in res]
@@ -261,7 +235,7 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
             prepared_query, prepared_query_parameters = SqlQueryHelper.prepare_limit_query(
                 datasource.query, datasource.parameters, offset, limit
             )
-        with self._get_cursor(database=datasource.database) as cursor:
+        with self._get_connection(database=datasource.database).cursor() as cursor:
             cursor.execute(prepared_query, prepared_query_parameters)
             result: pd.DataFrame = cursor.fetch_dataframe()
             if result is None:
@@ -339,7 +313,7 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
         return ConnectorStatus(status=True, details=self._get_details(3, True), error=None)
 
     def describe(self, data_source: RedshiftDataSource) -> dict[str, Any]:
-        with self._get_cursor(database=data_source.database) as cursor:
+        with self._get_connection(database=data_source.database).cursor() as cursor:
             cursor.execute(DESCRIBE_QUERY.format(column=data_source.query.replace(';', '')))
             res = cursor.description
         return {
@@ -347,39 +321,20 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
             for col in res
         }
 
-    def _table_infos_rows(self) -> list[tuple[str, str, str, str, str]]:
-        """Get rows of (database, schema, table name, column name, column type)"""
-        with self._get_cursor(database=self.default_database) as cursor:
-            res = cursor.execute(
-                """
-                SELECT
-                    table_catalog,
-                    table_schema,
-                    table_name,
-                    column_name,
-                    data_type
-                FROM
-                    information_schema.columns
-                WHERE
-                    table_schema NOT IN ('information_schema', 'pg_catalog')
-                ORDER BY
-                    table_schema,
-                    table_name,
-                    ordinal_position;
-                """
+    def _db_table_info_rows(self, database: str) -> list[tuple[str, str, str, str]]:
+        with self._get_connection(database).cursor() as cursor:
+            """Get rows of (schema, table name, column name, column type)"""
+            cursor.execute(
+                r"""SELECT "schemaname", "tablename", "column", "type" FROM PG_TABLE_DEF WHERE schemaname = 'public';"""
             )
-        return res.fetchall()
+            return cursor.fetchall()
 
-    def get_model(self) -> list[TableInfo]:
-        """Retrieves the database tree structure using current connection"""
+    def _db_tables_info(self, database: str) -> list[tuple[str, str, str, str, str]]:
+        """Get rows of (database, schema, table name, column name, column type)"""
         table_infos = []
-        for database, schema, table_name, column_name, column_type in self._table_infos_rows():
+        for schema, table_name, column_name, column_type in self._db_table_info_rows(database):
             for row in table_infos[::-1]:
-                if (
-                    row['database'] == database
-                    and row['schema'] == schema
-                    and row['name'] == table_name
-                ):
+                if row['schema'] == schema and row['name'] == table_name:
                     row['columns'].append({'name': column_name, 'type': column_type})
                     break
             else:
@@ -394,12 +349,20 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
                 )
         return table_infos
 
+    def get_model(self) -> list[TableInfo]:
+        """Retrieves the database tree structure using current connection"""
+        tables_info = []
+        for db in self.available_dbs:
+            with suppress(redshift_connector.OperationalError):
+                tables_info += self._db_tables_info(db)
+
+        return tables_info
+
     def get_model_with_info(self) -> tuple[list[TableInfo], dict]:
         """Retrieves the database tree structure using current connection"""
-        available_dbs = self._list_db_names()
         databases_tree = []
         failed_databases = []
-        for db in available_dbs:
+        for db in self.available_dbs:
             try:
                 databases_tree += self._list_tables_info(db)
             except redshift_connector.OperationalError:
@@ -412,7 +375,7 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
         return (tables_info, metadata)
 
     def _list_db_names(self) -> list[str]:
-        with self._get_cursor(database=self.default_database) as cursor:
+        with self._get_connection(database=self.default_database).cursor() as cursor:
             # redshift has a weird system db called padb_harvest duplicating the content of 'dev' database
             # https://bit.ly/3GQJCdy, we have to filter it
             cursor.execute(
@@ -420,12 +383,7 @@ class RedshiftConnector(ToucanConnector, DiscoverableConnector):
             )
             return [db_name for (db_name,) in cursor.fetchall()]
 
-    def _list_tables_info(self, database_name: str = None) -> list[tuple]:
-        connection = redshift_connector.connect(
-            **self._get_connection_params(
-                database=self.default_database if not database_name else database_name
-            )
-        )
-        with connection.cursor() as cursor:
+    def _list_tables_info(self, database: str) -> list[tuple]:
+        with self._get_cursor(database) as cursor:
             cursor.execute(build_database_model_extraction_query())
             return cursor.fetchall()
